#ifndef __GL_WINDOW_H__
#define __GL_WINDOW_H__

#include "SpaceShip.h"
#include <ngl/Camera.h>
#include <ngl/Colour.h>
#include <ngl/SpotLight.h>
#include <ngl/TransformStack.h>
#include <ngl/Obj.h>
// must be included after our stuff becuase GLEW needs to be first
#include <QtOpenGL>


class GLWindow : public QGLWidget
{
// must include this if you use Qt signals/slots
Q_OBJECT
public :
  //----------------------------------------------------------------------------------------------------------------------
  /// @brief Constructor for GLWindow
  /// @param [in] parent the parent window to create the GL context in
  //----------------------------------------------------------------------------------------------------------------------
  GLWindow(
           QWidget *_parent
          );
  //----------------------------------------------------------------------------------------------------------------------
	/// @brief dtor add any code here to tidy up
  //----------------------------------------------------------------------------------------------------------------------
  ~GLWindow();
  //----------------------------------------------------------------------------------------------------------------------
	/// @brief processKeyDown passed from the main window class
	/// @param *_event a pointer to the QKeyEvent passed from main window
	/// class
  //----------------------------------------------------------------------------------------------------------------------
	void processKeyDown(
									 		QKeyEvent *_event
										 );
  //----------------------------------------------------------------------------------------------------------------------
	/// @brief processKeyUp passed from the main window class
	/// @param *_event a pointer to the QKeyEvent passed from main window
	/// class
  //----------------------------------------------------------------------------------------------------------------------
	void processKeyUp(
										QKeyEvent *_event
										);

private :
  //----------------------------------------------------------------------------------------------------------------------
  /// @brief a timer triggered by the startTimer call in the ctor this is called
	/// to update the ship position based on the key presses
  //----------------------------------------------------------------------------------------------------------------------
	int m_updateShipTimer;

	int m_redrawTimer;
  //----------------------------------------------------------------------------------------------------------------------
  /// @brief Our Camera
  //----------------------------------------------------------------------------------------------------------------------
  ngl::Camera *m_cam;
  //----------------------------------------------------------------------------------------------------------------------
  /// @brief our transform stack for drawing the teapots
  //----------------------------------------------------------------------------------------------------------------------
  ngl::TransformStack m_transformStack;
  /// @brief a pointer to our spaceship
  SpaceShip *m_ship;
	/// @brief the keys being pressed
	QSet<Qt::Key> m_keysPressed;
protected:

  //----------------------------------------------------------------------------------------------------------------------
  /// @brief  The following methods must be implimented in the sub class
  /// this is called when the window is created
  //----------------------------------------------------------------------------------------------------------------------
  void initializeGL();

  //----------------------------------------------------------------------------------------------------------------------
  /// @brief this is called whenever the window is re-sized
  /// @param[in] _w the width of the resized window
  /// @param[in] _h the height of the resized window
  //----------------------------------------------------------------------------------------------------------------------
  void resizeGL(
                const int _w,
                const int _h
               );
  //----------------------------------------------------------------------------------------------------------------------
  /// @brief this is the main gl drawing routine which is called whenever the window needs to
  /// be re-drawn
  //----------------------------------------------------------------------------------------------------------------------
  void paintGL();

private :

   //----------------------------------------------------------------------------------------------------------------------
  /// @brief the timer event triggered from the timers
  /// @param _even the event of the timer triggered by Qt
  //----------------------------------------------------------------------------------------------------------------------
  void timerEvent(
                  QTimerEvent *_event
                 );
  //----------------------------------------------------------------------------------------------------------------------
  /// @brief method  to move the ship
  //----------------------------------------------------------------------------------------------------------------------
	void moveShip();
};

#endif
